@@ -5,7 +5,7 @@ use strict;
 
 use vars qw($VERSION @ISA);
 
-$VERSION = '0.02';
+$VERSION = '0.03';
 
 use Apache::Session;
 
@@ -24,7 +24,7 @@ Params::Validate::validation_options( on_fail => sub { param_error( join '', @_
 # This may change later
 @ISA = qw(HTML::Mason::Request);
 
-__PACKAGE__->valid_params
+my %params =
     ( session_always_write =>
       { type => BOOLEAN,
 	default => 1,
@@ -182,6 +182,8 @@ __PACKAGE__->valid_params
 	descr => 'Path used by Apache::Session::PHP' },
 
     );
+__PACKAGE__->valid_params(%params);
+
 
 # What set of parameters are required for each session class.
 # Multiple array refs represent multiple possible sets of parameters
@@ -267,12 +269,20 @@ $StudlyForm{textsize} = 'textsize';
 
 sub new
 {
-    @ISA = 
-        ( $HTML::Mason::ApacheHandler::VERSION ?
-          'HTML::Mason::Request::ApacheHandler' :
-          $HTML::Mason::CGIHandler::VERSION ?
-          'HTML::Mason::Request::CGI' :
-          'HTML::Mason::Request' );
+    {
+        my $new_isa = ( $HTML::Mason::ApacheHandler::VERSION ?
+                        'HTML::Mason::Request::ApacheHandler' :
+                        $HTML::Mason::CGIHandler::VERSION ?
+                        'HTML::Mason::Request::CGI' :
+                        'HTML::Mason::Request' );
+
+        if ( $ISA[0] ne $new_isa )
+        {
+            @ISA = $new_isa;
+        }
+
+        __PACKAGE__->valid_params(%params);
+    }
 
     my $class = shift;
 
@@ -372,7 +382,7 @@ sub session
 {
     my $self = shift;
 
-    return $self->parent->session if $self->is_subrequest;
+    return $self->parent_request->session(@_) if $self->is_subrequest;
 
     $self->{session} ||= $self->_make_session(@_);
     $self->{session_id} ||= $self->{session}{_session_id};
@@ -547,6 +557,8 @@ In a component:
 This module integrates C<Apache::Session> into Mason by adding methods
 to the Mason Request object available in all Mason components.
 
+Any subrequests created by a request share the same session hash.
+
 =head1 USAGE
 
 To use this module you need to tell Mason to use this class for
@@ -754,7 +766,7 @@ C<Apache::Session::Generate::ModUsertrack>.
 
 =item * session_save_path / MasonSessionSavePath  =>  path
 
-Corresponds to the C<ModSavePath> parameter given to
+Corresponds to the C<SavePath> parameter given to
 C<Apache::Session::PHP>.
 
 =back
